
#ifdef WIN32
  #define WINDOWS
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stddef.h>

#ifndef WINDOWS

#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

#endif
#ifdef WINDOWS

#include <windows.h>

#endif

#include "SDL.h"

#include "gfx.h"

typedef enum {
    HOBBIT_OUTPUT,
    HOBBIT_OUTPUT_SAMELINE,
    HOBBIT_PROMPT,
    HOBBIT_ANIMATION,
    HOBBIT_ERROR,
    HOBBIT_PAUSE,
    HOBBIT_CLEAR
} MsgType;

static void die(const char *err)
{
    perror(err);
    exit(-1);
}

void *xmalloc(int size)
{
    void *retv = malloc(size);
    if (!retv)
        die("malloc()");
    return retv;
}

void *xrealloc(void *p, int size)
{
    void *retv = realloc(p, size);
    if (!retv)
        die("realloc()");
    return retv;
}

#ifndef WINDOWS

static void xpipe(int* filedes)
{
    if (pipe(filedes) == -1)
        die("pipe()");
}

static void xclose(int fd)
{
    if (close(fd) == -1)
        die("close()");
}

static int xdup2(int oldfd, int newfd)
{
    int retv;
    if ((retv = dup2(oldfd, newfd)) == -1)
        die("dup2()");
    return retv;
}

FILE * xfdopen(int fd, const char *mode)
{
    FILE *retv = fdopen(fd, mode);
    if (!retv)
        die("fdopen()");
    return retv;
}

static pid_t ipc_open(FILE **inf, FILE **outf,
    const char *filename, char *const *args)
{
    pid_t pid;
    int out[2];
    int in[2];
    
    xpipe(in);
    xpipe(out);
    if ((pid = fork()) < 0)
        die("fork()");
    if (pid == 0) {
        /* child */
        xdup2(out[1], STDOUT_FILENO);
        xclose(out[1]);
        xclose(out[0]);
        xdup2(in[0], STDIN_FILENO);
        xclose(in[0]);
        xclose(in[1]);
        execv(filename, args);
        die("execv()");
    }
    
    /* parent */
    xclose(out[1]);
    xclose(in[0]);
    *inf = xfdopen(in[1], "w");
    *outf = xfdopen(out[0], "r");
    
    return pid;
}

FILE *backend_in;
FILE *backend_out;

static void start_backend(void)
{
    char *const args[] = {
        "backend",
        NULL
    };
    ipc_open(&backend_in, &backend_out, args[0], args);
}

static unsigned char *read_line_from_backend(void)
{
    int size = 100, l = 0;
    unsigned char *retv = xmalloc(size);
    for (;;) {
        int c = fgetc(backend_out);
        //fprintf(stderr, "char: %d '%c'\n", c, c);
        if (c == EOF) {
            if (ferror(backend_out))
                die("fgetc()");
            exit(-1);
        }
        retv[l++] = c;
        if (l >= size)
            retv = xrealloc(retv, (size *= 2));
        if (c == '\n') {
            retv[l-1] = '\0';
            //fprintf(stderr, "LINE: '%s'\n", retv);
            return retv;
        }
    }
}

static void write_to_backend(const unsigned char *s)
{
    //fprintf(stderr, "write_to_backend('%s')\n", s);
    int len = strlen((char*)s);
    if (len > fwrite(s, 1, len, backend_in))
        die("fwrite()");
    fflush(backend_in);
}

#endif
#ifdef WINDOWS

HANDLE hChildStdinRd, hChildStdinWr,
    hChildStdoutRd, hChildStdoutWr,
    hInputFile, hStdout;
 
BOOL CreateChildProcess(VOID);
VOID WriteToPipe(VOID);
VOID ReadFromPipe(VOID);
VOID ErrorExit(LPTSTR);
VOID ErrMsg(LPTSTR, BOOL); 

static void start_backend(void)
{
    SECURITY_ATTRIBUTES saAttr;
    BOOL fSuccess;
    
// Set the bInheritHandle flag so pipe handles are inherited.
    
    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    saAttr.bInheritHandle = TRUE;
    saAttr.lpSecurityDescriptor = NULL;
    
// Get the handle to the current STDOUT.
    
    hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
    
// Create a pipe for the child process's STDOUT.
    
    if (! CreatePipe(&hChildStdoutRd, &hChildStdoutWr, &saAttr, 0))
        ErrorExit("Stdout pipe creation failed\n");
    
// Ensure the read handle to the pipe for STDOUT is not inherited.
    
    SetHandleInformation( hChildStdoutRd, HANDLE_FLAG_INHERIT, 0);
    
// Create a pipe for the child process's STDIN.
    
    if (! CreatePipe(&hChildStdinRd, &hChildStdinWr, &saAttr, 0))
        ErrorExit("Stdin pipe creation failed\n");
    
// Ensure the write handle to the pipe for STDIN is not inherited.
    
    SetHandleInformation( hChildStdinWr, HANDLE_FLAG_INHERIT, 0);
    
// Now create the child process.
    
    fSuccess = CreateChildProcess();
    if (! fSuccess)
        ErrorExit("Create process failed with"); 
}

BOOL CreateChildProcess()
{
    TCHAR szCmdline[]=TEXT("backend.exe");
    PROCESS_INFORMATION piProcInfo;
    STARTUPINFO siStartInfo;
    BOOL bFuncRetn = FALSE;
    
// Set up members of the PROCESS_INFORMATION structure.
    
    ZeroMemory( &piProcInfo, sizeof(PROCESS_INFORMATION) );
    
// Set up members of the STARTUPINFO structure.
    
    ZeroMemory( &siStartInfo, sizeof(STARTUPINFO) );
    siStartInfo.cb = sizeof(STARTUPINFO);
    siStartInfo.hStdError = hChildStdoutWr;
    siStartInfo.hStdOutput = hChildStdoutWr;
    siStartInfo.hStdInput = hChildStdinRd;
    siStartInfo.dwFlags |= STARTF_USESTDHANDLES;
    
// Create the child process.
    
    bFuncRetn = CreateProcess(NULL,
        szCmdline,     // command line
        NULL,          // process security attributes
        NULL,          // primary thread security attributes
        TRUE,          // handles are inherited
        CREATE_NO_WINDOW,             // creation flags
        NULL,          // use parent's environment
        NULL,          // use parent's current directory
        &siStartInfo,  // STARTUPINFO pointer
        &piProcInfo);  // receives PROCESS_INFORMATION
    
    if (bFuncRetn == 0)
        ErrorExit("CreateProcess failed\n");
    else
    {
        CloseHandle(piProcInfo.hProcess);
        CloseHandle(piProcInfo.hThread);
        return bFuncRetn;
    }

    return 0;
}

static unsigned char *read_line_from_backend(void)
{
    int size = 100, l = 0;
    unsigned char *retv = xmalloc(size);
    for (;;) {
        char buf[1];
        DWORD dwRead;
        if (!ReadFile(hChildStdoutRd, buf, 1, &dwRead, NULL) || dwRead==0)
            die("ReadFile");
        int c = buf[0];
        
        retv[l++] = c;
        if (l >= size)
            retv = xrealloc(retv, (size *= 2));
        if (c == '\n') {
            retv[l-1] = '\0';
            //fprintf(stderr, "LINE: '%s'\n", retv);
            return retv;
        }
    }
}

static void write_to_backend(const unsigned char *s)
{
    DWORD dwWritten;
    WriteFile(hChildStdinWr, s, strlen(s), &dwWritten, NULL);
}

VOID ErrorExit (LPTSTR lpszMessage)
{
    fprintf(stderr, "%s\n", lpszMessage);
    ExitProcess(0);
}

#endif

unsigned char *parse_line(unsigned char *line, MsgType *type)
{
    //fprintf(stderr, "parse_line('%s')\n", line);
    struct {
        const char *s;
        MsgType type;
    } types[] = {
        { "[PROMPT]", HOBBIT_PROMPT },
        { "[ANIMATION]", HOBBIT_ANIMATION },
        { "[ERROR]", HOBBIT_ERROR },
        { "[PAUSE]", HOBBIT_PAUSE },
        { "[CLEAR]", HOBBIT_CLEAR },
        { "[SAMELINE]", HOBBIT_OUTPUT_SAMELINE }
    };
    int i;
    for (i = 0; i < sizeof(types)/sizeof(*types); ++i) {
        int len = strlen(types[i].s);
        if (0 == strncmp((char*)line, types[i].s, len)) {
            *type = types[i].type;
            return line + len;
        }
    }
    *type = HOBBIT_OUTPUT;
    return line;
}

int main(int argc, char **argv)
{
    int scale = argc > 1 ? atoi(argv[1]) : 3;
    unsigned char *line, *msg, *command;
    MsgType type;
    start_backend();
    gfx_init(scale);
    for (;;) {
        line = read_line_from_backend();
        msg = parse_line(line, &type);
        //fprintf(stderr, "msg: '%s'\n", msg);
        switch (type) {
        case HOBBIT_OUTPUT:
            gfx_out((unsigned char*)"\n");
            gfx_out(msg);
            break;
        case HOBBIT_OUTPUT_SAMELINE:
            gfx_out(msg);
            break;
        case HOBBIT_PROMPT:
            command = gfx_in(msg);
            write_to_backend(command);
            write_to_backend((unsigned char*)"\n");
            break;
        case HOBBIT_ANIMATION:
            gfx_animation(msg);
            break;
        case HOBBIT_ERROR:
            gfx_err(msg);
            gfx_err((unsigned char*)"\n");
            break;
        case HOBBIT_PAUSE:
            gfx_pause();
            break;
        case HOBBIT_CLEAR:
            gfx_clear_screen();
            break;
        }
        free(line);
    }
}
