
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"

#include "gfx.h"
#include "anim.h"
#include "font.h"

enum {
    SCREEN_WIDTH = 256,
    SCREEN_HEIGHT = 192,
    ANIM_HEIGHT = 128,
    HOBBIT_FONT_HEIGHT = 8,
    HOBBIT_FONT_WIDTH = 6,
    HOBBIT_FONT_MAX_CODE = 255,
    SPECTRUM_FONT_HEIGHT = 8,
    SPECTRUM_FONT_WIDTH = 8,
    SPECTRUM_FONT_MAX_CODE = 255,
    COMM_Y = 152,
    ADV_NUM_ROWS = 18,
    ADV_NUM_COLS = 42,
    COMM_NUM_ROWS = 5,
    COMM_NUM_COLS = 32,
    ORNAMENT_WIDTH = 16,
    ORNAMENT_HEIGHT = 8,
    ORNAMENT_Y = 144,
    MAX_INPUT_LENGTH = 128
};

typedef enum {
    BLACK = 128,
    BLUE = 129,
    RED = 130,
    MAGENTA = 131,
    GREEN = 132,
    CYAN = 133,
    YELLOW = 134,
    WHITE = 135
} Color;

enum {
    END_FRAME = 254,
    END_ANIM = 255,
    HLINE = 200
};

struct {
    Color c;
    int r;
    int g;
    int b;
} rgb[] = {
    { BLACK, 0, 0, 0 },
    { BLUE, 0, 0, 197 },
    { RED, 197, 0, 0 },
    { MAGENTA, 197, 0, 197 },
    { GREEN, 0, 194, 0 },
    { CYAN, 0, 194, 197 },
    { YELLOW, 197, 194, 0 },
    { WHITE, 197, 194, 197 }
};

SDL_Surface* screen;
SDL_Surface* hobbit_font;
SDL_Surface* spectrum_font;
SDL_Surface* ornament;
int scale;
int adv_cur = 0;
int comm_cur_x = 0;
int comm_cur_y = 0;
int fullscreen = 0;
int original_scale;

Color buffer[SCREEN_WIDTH][SCREEN_HEIGHT];

static Uint32 get_color(Color c, SDL_Surface* s)
{
    int i;
    for (i = 0; i < sizeof(rgb)/sizeof(*rgb); ++i) {
        if (rgb[i].c == c)
            return SDL_MapRGB(s->format, rgb[i].r, rgb[i].g, rgb[i].b);
    }
    fprintf(stderr, "Impossible happened.\n");
    exit(-1);
}

static void load_surfaces(void)
{
    int i, j, k, l;
    
    hobbit_font = SDL_CreateRGBSurface(SDL_SWSURFACE,
        HOBBIT_FONT_WIDTH * (HOBBIT_FONT_MAX_CODE + 1) * scale,
        HOBBIT_FONT_HEIGHT * scale,
        32, 0xff000000, 0x00ff0000, 0x0000ff00, 0);
    if (!hobbit_font) {
        fprintf(stderr, "CreateRGBSurface() failed: %s\n", SDL_GetError());
        exit(-1);
    }
    spectrum_font = SDL_CreateRGBSurface(SDL_SWSURFACE,
        SPECTRUM_FONT_WIDTH * (SPECTRUM_FONT_MAX_CODE + 1) * scale,
        SPECTRUM_FONT_HEIGHT * scale,
        32, 0xff000000, 0x00ff0000, 0x0000ff00, 0);
    if (!spectrum_font) {
        fprintf(stderr, "CreateRGBSurface() failed: %s\n", SDL_GetError());
        exit(-1);
    }
    ornament = SDL_CreateRGBSurface(SDL_SWSURFACE,
        ORNAMENT_WIDTH * scale, ORNAMENT_HEIGHT * scale,
        32, 0xff000000, 0x00ff0000, 0x0000ff00, 0);
    if (!ornament) {
        fprintf(stderr, "CreateRGBSurface() failed: %s\n", SDL_GetError());
        exit(-1);
    }
    i = 0;
    for (j = 0; j < HOBBIT_FONT_MAX_CODE; j++) {
        for (k = 0; k < HOBBIT_FONT_HEIGHT; k++) {
            for (l = 0; l < HOBBIT_FONT_WIDTH; l++) {
                Color c = hobbit_font_data[i] ? BLACK : WHITE;
                SDL_Rect dest;
                dest.x = (j * HOBBIT_FONT_WIDTH + l) * scale;
                dest.y = k * scale;
                dest.w = scale;
                dest.h = scale;
                if (-1 == SDL_FillRect(hobbit_font, &dest,
                        get_color(c, hobbit_font))) {
                    fprintf(stderr, "SDL_FillRect() failed: %s\n",
                        SDL_GetError());
                    exit(-1);
                }
                i++;
            }
        }
    }
    i = 0;
    for (j = 0; j < SPECTRUM_FONT_MAX_CODE; j++) {
        for (k = 0; k < SPECTRUM_FONT_HEIGHT; k++) {
            for (l = 0; l < SPECTRUM_FONT_WIDTH; l++) {
                Color c = spectrum_font_data[i] ? BLACK : WHITE;
                SDL_Rect dest;
                dest.x = (j * SPECTRUM_FONT_WIDTH + l) * scale;
                dest.y = k * scale;
                dest.w = scale;
                dest.h = scale;
                if (-1 == SDL_FillRect(spectrum_font, &dest,
                        get_color(c, spectrum_font))) {
                    fprintf(stderr, "SDL_FillRect() failed: %s\n",
                        SDL_GetError());
                    exit(-1);
                }
                i++;
            }
        }
    }
    i = 0;
    for (k = 0; k < ORNAMENT_HEIGHT; k++) {
        for (l = 0; l < ORNAMENT_WIDTH; l++) {
            Color c = ornament_data[i] ? BLACK : WHITE;
            SDL_Rect dest;
            dest.x = l * scale;
            dest.y = k * scale;
            dest.w = scale;
            dest.h = scale;
            if (-1 == SDL_FillRect(ornament, &dest,
                    get_color(c, ornament))) {
                fprintf(stderr, "SDL_FillRect() failed: %s\n",
                    SDL_GetError());
                exit(-1);
            }
            i++;
        }
    }
}

static void free_surfaces(void)
{
    SDL_FreeSurface(hobbit_font);
    SDL_FreeSurface(spectrum_font);
    SDL_FreeSurface(ornament);
}

static void set_video_mode(void)
{
    screen = SDL_SetVideoMode(SCREEN_WIDTH * scale, SCREEN_HEIGHT * scale,
        0, fullscreen ? SDL_FULLSCREEN : 0);
    if (!screen) {
        fprintf(stderr, "Unable to set video mode: %s\n", SDL_GetError());
        exit(-1);
    }
}

void gfx_init(int sc)
{
    if (0 != SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER)) {
        fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
        exit(-1);
    }
    atexit(SDL_Quit);
    SDL_WM_SetCaption("Hobbit", NULL);
    scale = sc ? sc : 4;
    original_scale = scale;
    fullscreen = (sc == 0);
    set_video_mode();
    load_surfaces();
    gfx_clear_screen();
}

static void copy_buffer(void)
{
    int i, j;
    for (i = 0; i < SCREEN_WIDTH; i++) {
        for (j = 0; j < SCREEN_HEIGHT; j++) {
            SDL_Rect dest;
            dest.x = i * scale;
            dest.y = j * scale;
            dest.w = scale;
            dest.h = scale;
            if (-1 == SDL_FillRect(screen, &dest,
                    get_color(buffer[i][j], screen))) {
                fprintf(stderr, "SDL_FillRect() failed: %s\n",
                    SDL_GetError());
                exit(-1);
            }
        }
    }
}

static void update(void)
{
    SDL_UpdateRect(screen, 0, 0, 0, 0);
}

static void gfx_toggle_fullscreen(void)
{
    fullscreen = !fullscreen;
    if (fullscreen) {
        original_scale = scale;
        if (scale != 4) {
            scale = 4;
            free_surfaces();
            load_surfaces();
        }
    } else {
        if (scale != original_scale) {
            scale = original_scale;
            free_surfaces();
            load_surfaces();
        }
    }
    set_video_mode();
    copy_buffer();
    update();
}

static void scale_changed(void)
{
    free_surfaces();
    load_surfaces();
    set_video_mode();
    copy_buffer();
    update();
}

static void gfx_scale_up(void)
{
    if (!fullscreen) {
        scale = scale % 5 + 1;
        scale_changed();
    }
}

static void gfx_scale_down(void)
{
    if (!fullscreen) {
        scale = (scale + 3) % 5 + 1;
        scale_changed();
    }
}

static void adv_print_at(int x, int y, int c)
{
    //fprintf(stderr, "adv_print_at(%d, %d, %d)\n", x, y, c);
    SDL_Rect src, dest;
    int i, j;
    src.x = c * HOBBIT_FONT_WIDTH * scale;
    src.y = 0;
    src.w = HOBBIT_FONT_WIDTH * scale;
    src.h = HOBBIT_FONT_HEIGHT * scale;
    dest.x = (1 + x * HOBBIT_FONT_WIDTH) * scale;
    dest.y = y * HOBBIT_FONT_HEIGHT * scale;
    SDL_BlitSurface(hobbit_font, &src, screen, &dest);
    for (i = 0; i < HOBBIT_FONT_WIDTH; i++) {
        for (j = 0; j < HOBBIT_FONT_HEIGHT; j++) {
            buffer[1 + x * HOBBIT_FONT_WIDTH + i][y * HOBBIT_FONT_HEIGHT + j]
                = hobbit_font_data[c * HOBBIT_FONT_WIDTH * HOBBIT_FONT_HEIGHT
                    + j * HOBBIT_FONT_WIDTH + i] ? BLACK : WHITE;
        }
    }
}

static void comm_print_at(int x, int y, int c)
{
    //fprintf(stderr, "comm_print_at(%d, %d, %d)\n", x, y, c);
    SDL_Rect src, dest;
    int i, j;
    src.x = c * SPECTRUM_FONT_WIDTH * scale;
    src.y = 0;
    src.w = SPECTRUM_FONT_WIDTH * scale;
    src.h = SPECTRUM_FONT_HEIGHT * scale;
    dest.x = x * SPECTRUM_FONT_WIDTH * scale;
    dest.y = (COMM_Y + y * SPECTRUM_FONT_HEIGHT) * scale;
    SDL_BlitSurface(spectrum_font, &src, screen, &dest);
    for (i = 0; i < SPECTRUM_FONT_WIDTH; i++) {
        for (j = 0; j < SPECTRUM_FONT_HEIGHT; j++) {
            buffer[x * SPECTRUM_FONT_WIDTH + i]
                [COMM_Y + y * SPECTRUM_FONT_HEIGHT + j]
                = spectrum_font_data
                [c * SPECTRUM_FONT_WIDTH * SPECTRUM_FONT_HEIGHT
                    + j * SPECTRUM_FONT_WIDTH + i] ? BLACK : WHITE;
        }
    }
}

static void draw_ornament(void)
{
    int i, j, k;
    int n = SCREEN_WIDTH / ORNAMENT_WIDTH;
    SDL_Rect src, dest;
    src.x = 0;
    src.y = 0;
    src.w = ORNAMENT_WIDTH * scale;
    src.h = ORNAMENT_HEIGHT * scale;
    dest.y = ORNAMENT_Y * scale;
    for (i = 0; i < n; i++) {
        dest.x = i * ORNAMENT_WIDTH * scale;
        SDL_BlitSurface(ornament, &src, screen, &dest);
        for (j = 0; j < ORNAMENT_WIDTH; j++) {
            for (k = 0; k < ORNAMENT_HEIGHT; k++) {
                buffer[i * ORNAMENT_WIDTH + j][ORNAMENT_Y + k]
                    = ornament_data[k * ORNAMENT_WIDTH + j] ? BLACK : WHITE;
            }
        }
    }
}

void gfx_clear_screen(void)
{
    int i, j;
    SDL_Rect dest;
    dest.x = 0;
    dest.y = 0;
    dest.w = SCREEN_WIDTH * scale;
    dest.h = SCREEN_HEIGHT * scale;
    if (-1 == SDL_FillRect(screen, &dest, get_color(WHITE, screen))) {
        fprintf(stderr, "SDL_FillRect() failed: %s\n", SDL_GetError());
        exit(-1);
    }
    for (i = 0; i < SCREEN_WIDTH; i++)
        for (j = 0; j < SCREEN_HEIGHT; j++)
            buffer[i][j] = WHITE;
    draw_ornament();
    adv_cur = 0;
    comm_cur_x = 0;
    comm_cur_y = 0;
    update();
}

static void adv_scroll(void)
{
    int i, j, k;
    SDL_Rect src, dest;
    src.x = 0;
    src.w = SCREEN_WIDTH * scale;
    src.h = HOBBIT_FONT_HEIGHT * scale;
    dest.x = 0;
    for (i = 0; i < ADV_NUM_ROWS - 1; ++i) {
        dest.y = i * HOBBIT_FONT_HEIGHT * scale;
        src.y = dest.y + HOBBIT_FONT_HEIGHT * scale;
        SDL_BlitSurface(screen, &src, screen, &dest);
        for (j = 0; j < SCREEN_WIDTH; j++) {
            for (k = 0; k < HOBBIT_FONT_HEIGHT; k++) {
                buffer[j][i * HOBBIT_FONT_HEIGHT + k]
                    = buffer[j][(i + 1) * HOBBIT_FONT_HEIGHT + k];
            }
        }
    }
    for (i = 0; i < ADV_NUM_COLS; ++i)
        adv_print_at(i, ADV_NUM_ROWS - 1, ' ');
}

static void comm_scroll(void)
{
    int i, j, k;
    SDL_Rect src, dest;
    src.x = 0;
    src.w = SCREEN_WIDTH * scale;
    src.h = SPECTRUM_FONT_HEIGHT * scale;
    dest.x = 0;
    for (i = 0; i < COMM_NUM_ROWS - 1; ++i) {
        dest.y = (COMM_Y + i * SPECTRUM_FONT_HEIGHT) * scale;
        src.y = dest.y + SPECTRUM_FONT_HEIGHT * scale;
        SDL_BlitSurface(screen, &src, screen, &dest);
        for (j = 0; j < SCREEN_WIDTH; j++) {
            for (k = 0; k < SPECTRUM_FONT_HEIGHT; k++) {
                buffer[j][COMM_Y + i * SPECTRUM_FONT_HEIGHT + k]
                    = buffer[j][COMM_Y + (i + 1) * SPECTRUM_FONT_HEIGHT + k];
            }
        }
    }
    for (i = 0; i < COMM_NUM_COLS; ++i)
        comm_print_at(i, COMM_NUM_ROWS - 1, ' ');
}

static int simple_keyboard_event(SDL_Event* e)
{
    int sym = e->key.keysym.sym;
    //int mod = e->key.keysym.mod;
    //fprintf(stderr, "sym: %d, mod: %d\n", sym, mod);
    if (sym == SDLK_F2) {
        gfx_toggle_fullscreen();
        return 1;
    } else if (sym == SDLK_F4) {
        gfx_scale_up();
        return 1;
    } else if (sym == SDLK_F3) {
        gfx_scale_down();
        return 1;
    } else if (sym == SDLK_ESCAPE) {
        exit(0);
    }
    return 0;
}

static void process_simple_events(void)
{
    SDL_Event e;
    while(SDL_PollEvent(&e)) {
        switch (e.type) {
        case SDL_QUIT:
            exit(0);
        case SDL_KEYDOWN:
            simple_keyboard_event(&e);
            break;
        default:
            break;
        }
    }
}

void gfx_out(const unsigned char* s)
{
    //fprintf(stderr, "gfx_out('%s')\n", s);
    const unsigned char* p = s;
    while (*p) {
        int code = *p;
        //fprintf(stderr, "code: '%c' %d\n", code, code);
        if (code > HOBBIT_FONT_MAX_CODE)
            code = '?';
        if (code == '\n'  ||  adv_cur == ADV_NUM_COLS) {
            adv_scroll();
            adv_cur = 0;
        }
        if (code != '\n') {
            adv_print_at(adv_cur, ADV_NUM_ROWS - 1, code);
            ++adv_cur;
        }
        ++p;
    }
    update();
    SDL_Delay(100);
    process_simple_events();
}

static void comm_back(void)
{
    if (comm_cur_x > 0) {
        --comm_cur_x;
    } else {
        --comm_cur_y;
        comm_cur_x = COMM_NUM_COLS - 1;
    }
}

static void comm_print(const unsigned char* s)
{
    const unsigned char* p = s;
    while (*p) {
        int code = *p;
        if (code > SPECTRUM_FONT_MAX_CODE)
            code = '?';
        if (code == '\n'  ||  comm_cur_x == COMM_NUM_COLS) {
            if (comm_cur_y == COMM_NUM_ROWS - 1) {
                comm_scroll();
            } else {
                ++comm_cur_y;
            }
            comm_cur_x = 0;
        }
        if (code != '\n') {
            comm_print_at(comm_cur_x, comm_cur_y, code);
            ++comm_cur_x;
        }
        ++p;
    }
}

void gfx_err(const unsigned char* s)
{
    comm_print(s);
    update();
}

static int alter_sym(int sym)
{
    switch (sym) {
    case 'a': return 161;
    case 'e': return 202;
    case 's': return 166;
    case 'c': return 198;
    case 'n': return 209;
    case 'z': return 175;
    case 'x': return 172;
    case 'o': return 211;
    case 'l': return 163;
    default: return 0;
    }
}

Uint32 timer_autowait(Uint32 interval, void *param)
{
    SDL_Event event;
    SDL_UserEvent userevent;
    
    userevent.type = SDL_USEREVENT;
    userevent.code = 0;
    userevent.data1 = NULL;
    userevent.data2 = NULL;
    
    event.type = SDL_USEREVENT;
    event.user = userevent;
    
    SDL_PushEvent(&event);
    return 0;
}

unsigned char* gfx_in(const unsigned char* prompt)
{
    static char retv[MAX_INPUT_LENGTH + 1];
    char old[MAX_INPUT_LENGTH + 1];
    char cs[2] = {0};
    int cur = 0;
    SDL_Event e;
    strcpy(old, retv);
    memset(retv, 0, sizeof(retv));
    comm_print(prompt);
    for (;;) {
        comm_print((unsigned char*)"+");
        comm_back();
        if (0 == SDL_PollEvent(NULL))
            update();
        SDL_TimerID timer = SDL_AddTimer(25000, timer_autowait, NULL);
        if (!timer) {
            fprintf(stderr, "SDL_AddTimer() failed: %s\n",
                SDL_GetError());
            exit(1);
        }
        if (0 == SDL_WaitEvent(&e)) {
            fprintf(stderr, "SDL_WaitEvent() failed: %s\n",
                SDL_GetError());
            exit(1);
        }
        SDL_RemoveTimer(timer);
        //fprintf(stderr, "Event\n");
        int c = 0;
        int sym, mod;
        int alts = 0;
        char* p;
        switch (e.type) {
        case SDL_QUIT:
            exit(0);
        case SDL_USEREVENT:
            for (p = retv; *p; p++) {
                *p = '\0';
                comm_print((unsigned char*)" ");
                comm_back();
                comm_back();
            }
            comm_print((unsigned char*)"CZEKAJ\n+");
            comm_back();
            strcpy(retv, "CZEKAJ");
            return (unsigned char*)retv;
        case SDL_KEYDOWN:
            if (simple_keyboard_event(&e))
                break;
            sym = e.key.keysym.sym;
            mod = e.key.keysym.mod;
            if (mod & KMOD_RALT || mod & KMOD_MODE)
                alts = alter_sym(sym);
            if ((mod & KMOD_LSHIFT || mod & KMOD_RSHIFT)
                && sym == SDLK_2 && !*retv) {
                strcpy(retv, old);
                comm_print((unsigned char*)" ");
                comm_back();
                comm_back();
                comm_back();
                comm_print((unsigned char*)"+");
                comm_back();
                return (unsigned char*)retv;
            }
            if (sym == SDLK_RETURN) {
                comm_print((unsigned char*)" \n+");
                comm_back();
                update();
                return (unsigned char*)retv;
            } else if (sym == SDLK_LEFT
                || sym == SDLK_DELETE  ||  sym == SDLK_BACKSPACE) {
                if (cur > 0) {
                    comm_print((unsigned char*)" ");
                    comm_back();
                    comm_back();
                    retv[--cur] = '\0';
                }
            } else if (sym == SDLK_u
                && (mod & KMOD_RCTRL || mod & KMOD_LCTRL)) {
                while (cur > 0) {
                    comm_print((unsigned char*)" ");
                    comm_back();
                    comm_back();
                    retv[--cur] = '\0';
                }
                update();
            } else if (sym == SDLK_PERIOD) {
                c = '.';
            } else if (sym == SDLK_SPACE) {
                c = ' ';
            } else if (sym == SDLK_QUOTE  ||  sym == SDLK_QUOTEDBL) {
                c = '"';
            } else if (alts != 0) {
                c = alts;
            } else if (sym >= 'a'  &&  sym <= 'z') {
                c = sym - ('a' - 'A');
            }
            if (c != 0  &&  cur < MAX_INPUT_LENGTH) {
                cs[0] = c;
                comm_print((unsigned char*)cs);
                retv[cur++] = c;
            }
            break;
        default:
            break;
        }
    }
}

void gfx_pause(void)
{
    SDL_Event e;
    for (;;) {
        if (0 == SDL_WaitEvent(&e)) {
            fprintf(stderr, "Error.\n");
            exit(1);
        }
        switch (e.type) {
        case SDL_QUIT:
            exit(0);
        case SDL_KEYDOWN:
            if (!simple_keyboard_event(&e))
                return;
            break;
        default:
            break;
        }
    }
}

void gfx_animation(const unsigned char* loc)
{
    const unsigned char* data = 0;
    const unsigned char* p;
    int i, j;
    for (i = 0; animations[i].key; ++i) {
        if (0 == strcmp(animations[i].key, (char*)loc)) {
            data = animations[i].data;
            break;
        }
    }
    if (!data) {
        fprintf(stderr, "ERROR: missing animation `%s'\n", loc);
        exit(-1);
    }
    SDL_Rect dest;
    dest.x = 0;
    dest.y = 0;
    dest.w = SCREEN_WIDTH * scale;
    dest.h = ANIM_HEIGHT * scale;
    if (-1 == SDL_FillRect(screen, &dest, get_color(data[0], screen))) {
        fprintf(stderr, "SDL_FillRect() failed: %s\n", SDL_GetError());
        exit(-1);
    }
    for (i = 0; i < SCREEN_WIDTH; i++)
        for (j = 0; j < ANIM_HEIGHT; j++)
            buffer[i][j] = data[0];
    update();
    Color currcolor = BLACK;
    int x1 = -1, y1 = -1, x2 = -1, y2 = -1;
    int start = SDL_GetTicks();
    for (p = data + 1; *p != END_ANIM; ++p) {
        if (*p == END_FRAME) {
            process_simple_events();
            if (x1 != -1) {
                int x = x1 * scale;
                int y = y1 * scale;
                int w = (x2 - x1 + 1) * scale;
                int h = (y2 - y1 + 1) * scale;
                SDL_UpdateRect(screen, x, y, w, h);
            }
            x1 = y1 = x2 = y2 = -1;
            int stop = SDL_GetTicks();
            int dt = stop - start;
            if (dt < 16)
                SDL_Delay(16 - dt);
            start = SDL_GetTicks();
            continue;
        }
        int rx1 = -1, rx2, ry1, ry2;
        if (*p == HLINE) {
            p++;
            ry1 = ry2 = *p++;
            rx1 = *p++;
            rx2 = *p;
        } else if (*p < 128) {
            ry1 = ry2 = *p++;
            rx1 = rx2 = *p;
        }
        if (rx1 != -1) {
            if (x1 == -1) {
                x1 = rx1;
                x2 = rx2;
                y1 = ry1;
                y2 = ry2;
            } else {
                if (rx1 < x1)
                    x1 = rx1;
                if (rx2 > x2)
                    x2 = rx2;
                if (ry1 < y1)
                    y1 = ry1;
                if (ry2 > y2)
                    y2 = ry2;
            }
            dest.x = rx1 * scale;
            dest.y = ry1 * scale;
            dest.w = (rx2 - rx1 + 1) * scale;
            dest.h = (ry2 - ry1 + 1) * scale;
            if (-1 == SDL_FillRect(screen, &dest,
                    get_color(currcolor, screen))) {
                fprintf(stderr, "SDL_FillRect() failed: %s\n",
                    SDL_GetError());
                exit(-1);
            }
            for (i = rx1; i <= rx2; i++)
                for (j = ry1; j <= ry2; j++)
                    buffer[i][j] = currcolor;
            
            continue;
        }
        currcolor = *p;
    }
    gfx_pause();
}
